/****************************************************************************
** Meta object code from reading C++ file 'AnnotationSettingsAdapter.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../kImageAnnotator/src/gui/annotator/settings/AnnotationSettingsAdapter.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AnnotationSettingsAdapter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15kImageAnnotator25AnnotationSettingsAdapterE_t {};
} // unnamed namespace

template <> constexpr inline auto kImageAnnotator::AnnotationSettingsAdapter::qt_create_metaobjectdata<qt_meta_tag_ZN15kImageAnnotator25AnnotationSettingsAdapterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "kImageAnnotator::AnnotationSettingsAdapter",
        "effectChanged",
        "",
        "ImageEffects",
        "effect",
        "zoomValueChanged",
        "value",
        "toolTypeChanged",
        "Tools",
        "toolType",
        "loadFromConfig",
        "tool",
        "loadFromItem",
        "const AbstractAnnotationItem*",
        "item",
        "toolColorChanged",
        "QColor",
        "color",
        "toolTextColorChanged",
        "toolWidthChanged",
        "width",
        "toolFillTypeChanged",
        "FillModes",
        "fill",
        "notifyNumberToolSeedChanged",
        "newNumberToolSeed",
        "obfuscateFactorChanged",
        "factor",
        "stickerChanged",
        "sticker",
        "shadowEnabledChanged",
        "enabled",
        "fontChanged",
        "QFont",
        "font",
        "scalingChanged",
        "scaling",
        "opacityChanged",
        "opacity",
        "fitImageToCurrentView"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'effectChanged'
        QtMocHelpers::SlotData<void(ImageEffects)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'zoomValueChanged'
        QtMocHelpers::SlotData<void(double)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 6 },
        }}),
        // Slot 'toolTypeChanged'
        QtMocHelpers::SlotData<void(Tools)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'loadFromConfig'
        QtMocHelpers::SlotData<void(Tools)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 11 },
        }}),
        // Slot 'loadFromItem'
        QtMocHelpers::SlotData<void(const AbstractAnnotationItem *)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'toolColorChanged'
        QtMocHelpers::SlotData<void(const QColor &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'toolTextColorChanged'
        QtMocHelpers::SlotData<void(const QColor &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'toolWidthChanged'
        QtMocHelpers::SlotData<void(int)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 20 },
        }}),
        // Slot 'toolFillTypeChanged'
        QtMocHelpers::SlotData<void(FillModes)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'notifyNumberToolSeedChanged'
        QtMocHelpers::SlotData<void(int)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'obfuscateFactorChanged'
        QtMocHelpers::SlotData<void(int)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'stickerChanged'
        QtMocHelpers::SlotData<void(const QString &)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 29 },
        }}),
        // Slot 'shadowEnabledChanged'
        QtMocHelpers::SlotData<void(bool)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 31 },
        }}),
        // Slot 'fontChanged'
        QtMocHelpers::SlotData<void(const QFont &)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Slot 'scalingChanged'
        QtMocHelpers::SlotData<void(qreal)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QReal, 36 },
        }}),
        // Slot 'opacityChanged'
        QtMocHelpers::SlotData<void(qreal)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QReal, 38 },
        }}),
        // Slot 'fitImageToCurrentView'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AnnotationSettingsAdapter, qt_meta_tag_ZN15kImageAnnotator25AnnotationSettingsAdapterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject kImageAnnotator::AnnotationSettingsAdapter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator25AnnotationSettingsAdapterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator25AnnotationSettingsAdapterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15kImageAnnotator25AnnotationSettingsAdapterE_t>.metaTypes,
    nullptr
} };

void kImageAnnotator::AnnotationSettingsAdapter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AnnotationSettingsAdapter *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->effectChanged((*reinterpret_cast<std::add_pointer_t<ImageEffects>>(_a[1]))); break;
        case 1: _t->zoomValueChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->toolTypeChanged((*reinterpret_cast<std::add_pointer_t<Tools>>(_a[1]))); break;
        case 3: _t->loadFromConfig((*reinterpret_cast<std::add_pointer_t<Tools>>(_a[1]))); break;
        case 4: _t->loadFromItem((*reinterpret_cast<std::add_pointer_t<const AbstractAnnotationItem*>>(_a[1]))); break;
        case 5: _t->toolColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 6: _t->toolTextColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 7: _t->toolWidthChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->toolFillTypeChanged((*reinterpret_cast<std::add_pointer_t<FillModes>>(_a[1]))); break;
        case 9: _t->notifyNumberToolSeedChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->obfuscateFactorChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->stickerChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->shadowEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->fontChanged((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 14: _t->scalingChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 15: _t->opacityChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 16: _t->fitImageToCurrentView(); break;
        default: ;
        }
    }
}

const QMetaObject *kImageAnnotator::AnnotationSettingsAdapter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *kImageAnnotator::AnnotationSettingsAdapter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator25AnnotationSettingsAdapterE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "AbstractSettingsProvider"))
        return static_cast< AbstractSettingsProvider*>(this);
    return QObject::qt_metacast(_clname);
}

int kImageAnnotator::AnnotationSettingsAdapter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    return _id;
}
QT_WARNING_POP
