/*
 * Copyright (C) 2018 Damir Porobic <damir.porobic@gmx.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef KIMAGEANNOTATOR_ANNOTATIONCONSTANTS_H
#define KIMAGEANNOTATOR_ANNOTATIONCONSTANTS_H

#include <QSize>
#include <QColor>

namespace kImageAnnotator {

inline namespace Constants {

const int MinPathResizeRectSize = 20;

const int ResizeHandleSize = 16;

const QSize SettingsWidgetSize(64, 32);

const QSize ToolButtonIconSize(28, 28);

const QSize MenuItemIconSize(20, 20);

const QSize SettingsWidgetIconSize(20, 20);

const QColor HoverColor(173, 216, 230); // "#add8e6"

} // namespace Constants

} // namespace kImageAnnotator

#endif //KIMAGEANNOTATOR_ANNOTATIONCONSTANTS_H
