/****************************************************************************
** Meta object code from reading C++ file 'KImageAnnotator.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../kImageAnnotator/include/kImageAnnotator/KImageAnnotator.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'KImageAnnotator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15kImageAnnotator15KImageAnnotatorE_t {};
} // unnamed namespace

template <> constexpr inline auto kImageAnnotator::KImageAnnotator::qt_create_metaobjectdata<qt_meta_tag_ZN15kImageAnnotator15KImageAnnotatorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "kImageAnnotator::KImageAnnotator",
        "imageChanged",
        "",
        "currentTabChanged",
        "index",
        "tabCloseRequested",
        "tabMoved",
        "fromIndex",
        "toIndex",
        "tabContextMenuOpened",
        "loadImage",
        "QPixmap",
        "pixmap",
        "addTab",
        "title",
        "toolTip",
        "updateTabInfo",
        "insertImageItem",
        "QPointF",
        "position",
        "setTextFont",
        "QFont",
        "font",
        "setNumberFont",
        "setItemShadowEnabled",
        "enabled",
        "setSmoothPathEnabled",
        "setSaveToolSelection",
        "setSmoothFactor",
        "factor",
        "setSwitchToSelectToolAfterDrawingItem",
        "setNumberToolSeedChangeUpdatesAllItems",
        "setTabBarAutoHide",
        "removeTab",
        "setStickers",
        "stickerPaths",
        "keepDefault",
        "addTabContextMenuActions",
        "QList<QAction*>",
        "actions",
        "setSettingsCollapsed",
        "isCollapsed",
        "setCanvasColor",
        "QColor",
        "color",
        "setSelectItemAfterDrawing",
        "setControlsWidgetVisible"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'imageChanged'
        QtMocHelpers::SignalData<void() const>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentTabChanged'
        QtMocHelpers::SignalData<void(int) const>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'tabCloseRequested'
        QtMocHelpers::SignalData<void(int) const>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'tabMoved'
        QtMocHelpers::SignalData<void(int, int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 },
        }}),
        // Signal 'tabContextMenuOpened'
        QtMocHelpers::SignalData<void(int) const>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'loadImage'
        QtMocHelpers::SlotData<void(const QPixmap &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'addTab'
        QtMocHelpers::SlotData<int(const QPixmap &, const QString &, const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 11, 12 }, { QMetaType::QString, 14 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'updateTabInfo'
        QtMocHelpers::SlotData<void(int, const QString &, const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 }, { QMetaType::QString, 14 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'insertImageItem'
        QtMocHelpers::SlotData<void(const QPointF &, const QPixmap &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { 0x80000000 | 11, 12 },
        }}),
        // Slot 'setTextFont'
        QtMocHelpers::SlotData<void(const QFont &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'setNumberFont'
        QtMocHelpers::SlotData<void(const QFont &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'setItemShadowEnabled'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Slot 'setSmoothPathEnabled'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Slot 'setSaveToolSelection'
        QtMocHelpers::SlotData<void(bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Slot 'setSmoothFactor'
        QtMocHelpers::SlotData<void(int)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 29 },
        }}),
        // Slot 'setSwitchToSelectToolAfterDrawingItem'
        QtMocHelpers::SlotData<void(bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Slot 'setNumberToolSeedChangeUpdatesAllItems'
        QtMocHelpers::SlotData<void(bool)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Slot 'setTabBarAutoHide'
        QtMocHelpers::SlotData<void(bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Slot 'removeTab'
        QtMocHelpers::SlotData<void(int)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'setStickers'
        QtMocHelpers::SlotData<void(const QStringList &, bool)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 35 }, { QMetaType::Bool, 36 },
        }}),
        // Slot 'addTabContextMenuActions'
        QtMocHelpers::SlotData<void(const QList<QAction*> &)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Slot 'setSettingsCollapsed'
        QtMocHelpers::SlotData<void(bool)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 41 },
        }}),
        // Slot 'setCanvasColor'
        QtMocHelpers::SlotData<void(const QColor &)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 43, 44 },
        }}),
        // Slot 'setSelectItemAfterDrawing'
        QtMocHelpers::SlotData<void(bool)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Slot 'setControlsWidgetVisible'
        QtMocHelpers::SlotData<void(bool)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KImageAnnotator, qt_meta_tag_ZN15kImageAnnotator15KImageAnnotatorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject kImageAnnotator::KImageAnnotator::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator15KImageAnnotatorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator15KImageAnnotatorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15kImageAnnotator15KImageAnnotatorE_t>.metaTypes,
    nullptr
} };

void kImageAnnotator::KImageAnnotator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KImageAnnotator *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->imageChanged(); break;
        case 1: _t->currentTabChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->tabCloseRequested((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->tabMoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->tabContextMenuOpened((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->loadImage((*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[1]))); break;
        case 6: { int _r = _t->addTab((*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->updateTabInfo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 8: _t->insertImageItem((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[2]))); break;
        case 9: _t->setTextFont((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 10: _t->setNumberFont((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 11: _t->setItemShadowEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->setSmoothPathEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->setSaveToolSelection((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->setSmoothFactor((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->setSwitchToSelectToolAfterDrawingItem((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->setNumberToolSeedChangeUpdatesAllItems((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->setTabBarAutoHide((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->removeTab((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 19: _t->setStickers((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 20: _t->addTabContextMenuActions((*reinterpret_cast<std::add_pointer_t<QList<QAction*>>>(_a[1]))); break;
        case 21: _t->setSettingsCollapsed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->setCanvasColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 23: _t->setSelectItemAfterDrawing((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->setControlsWidgetVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QAction*> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KImageAnnotator::*)() const>(_a, &KImageAnnotator::imageChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KImageAnnotator::*)(int ) const>(_a, &KImageAnnotator::currentTabChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KImageAnnotator::*)(int ) const>(_a, &KImageAnnotator::tabCloseRequested, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KImageAnnotator::*)(int , int )>(_a, &KImageAnnotator::tabMoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KImageAnnotator::*)(int ) const>(_a, &KImageAnnotator::tabContextMenuOpened, 4))
            return;
    }
}

const QMetaObject *kImageAnnotator::KImageAnnotator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *kImageAnnotator::KImageAnnotator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator15KImageAnnotatorE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int kImageAnnotator::KImageAnnotator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void kImageAnnotator::KImageAnnotator::imageChanged()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::KImageAnnotator *>(this), &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void kImageAnnotator::KImageAnnotator::currentTabChanged(int _t1)const
{
    QMetaObject::activate<void>(const_cast< kImageAnnotator::KImageAnnotator *>(this), &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void kImageAnnotator::KImageAnnotator::tabCloseRequested(int _t1)const
{
    QMetaObject::activate<void>(const_cast< kImageAnnotator::KImageAnnotator *>(this), &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void kImageAnnotator::KImageAnnotator::tabMoved(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void kImageAnnotator::KImageAnnotator::tabContextMenuOpened(int _t1)const
{
    QMetaObject::activate<void>(const_cast< kImageAnnotator::KImageAnnotator *>(this), &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
