/*
 * Copyright (C) 2021 Damir Porobic <damir.porobic@gmx.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "AnnotationGeneralSettings.h"

namespace kImageAnnotator {

AnnotationGeneralSettings::AnnotationGeneralSettings() :
	mMainLayout(new QBoxLayout(QBoxLayout::LeftToRight)),
	mZoomPicker(new ZoomPicker(this))
{
	initGui();
}

AnnotationGeneralSettings::~AnnotationGeneralSettings()
{
	delete mZoomPicker;
}

void AnnotationGeneralSettings::initGui()
{
	mMainLayout->addWidget(mZoomPicker);
	mMainLayout->setContentsMargins(3, 0, 3, 0);

	addExpandingWidget(mZoomPicker);

	setLayout(mMainLayout);

	setFocusPolicy(Qt::ClickFocus);

	connect(mZoomPicker, &ZoomPicker::zoomValueChanged, this, &AnnotationGeneralSettings::zoomValueChanged);
	connect(mZoomPicker, &ZoomPicker::fitImageToView, this, &AnnotationGeneralSettings::fitImageToCurrentView);
}

void AnnotationGeneralSettings::updateZoomLevel(double value)
{
	mZoomPicker->setZoomValue(value);
}

QString AnnotationGeneralSettings::name() const
{
	return tr("General Settings");
}

} // namespace kImageAnnotator
