/****************************************************************************
** Meta object code from reading C++ file 'Controls.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../kImageAnnotator/src/widgets/Controls.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Controls.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15kImageAnnotator8ControlsE_t {};
} // unnamed namespace

template <> constexpr inline auto kImageAnnotator::Controls::qt_create_metaobjectdata<qt_meta_tag_ZN15kImageAnnotator8ControlsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "kImageAnnotator::Controls",
        "undo",
        "",
        "redo",
        "crop",
        "scale",
        "rotate",
        "modifyCanvas",
        "cut"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'undo'
        QtMocHelpers::SignalData<void() const>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'redo'
        QtMocHelpers::SignalData<void() const>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'crop'
        QtMocHelpers::SignalData<void() const>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'scale'
        QtMocHelpers::SignalData<void() const>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'rotate'
        QtMocHelpers::SignalData<void() const>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'modifyCanvas'
        QtMocHelpers::SignalData<void() const>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cut'
        QtMocHelpers::SignalData<void() const>(8, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Controls, qt_meta_tag_ZN15kImageAnnotator8ControlsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject kImageAnnotator::Controls::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator8ControlsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator8ControlsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15kImageAnnotator8ControlsE_t>.metaTypes,
    nullptr
} };

void kImageAnnotator::Controls::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Controls *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->undo(); break;
        case 1: _t->redo(); break;
        case 2: _t->crop(); break;
        case 3: _t->scale(); break;
        case 4: _t->rotate(); break;
        case 5: _t->modifyCanvas(); break;
        case 6: _t->cut(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Controls::*)() const>(_a, &Controls::undo, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Controls::*)() const>(_a, &Controls::redo, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Controls::*)() const>(_a, &Controls::crop, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Controls::*)() const>(_a, &Controls::scale, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Controls::*)() const>(_a, &Controls::rotate, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Controls::*)() const>(_a, &Controls::modifyCanvas, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Controls::*)() const>(_a, &Controls::cut, 6))
            return;
    }
}

const QMetaObject *kImageAnnotator::Controls::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *kImageAnnotator::Controls::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator8ControlsE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int kImageAnnotator::Controls::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void kImageAnnotator::Controls::undo()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::Controls *>(this), &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void kImageAnnotator::Controls::redo()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::Controls *>(this), &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void kImageAnnotator::Controls::crop()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::Controls *>(this), &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void kImageAnnotator::Controls::scale()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::Controls *>(this), &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void kImageAnnotator::Controls::rotate()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::Controls *>(this), &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void kImageAnnotator::Controls::modifyCanvas()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::Controls *>(this), &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void kImageAnnotator::Controls::cut()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::Controls *>(this), &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
