/****************************************************************************
** Meta object code from reading C++ file 'AnnotationWidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../kImageAnnotator/src/gui/annotator/AnnotationWidget.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AnnotationWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15kImageAnnotator16AnnotationWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto kImageAnnotator::AnnotationWidget::qt_create_metaobjectdata<qt_meta_tag_ZN15kImageAnnotator16AnnotationWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "kImageAnnotator::AnnotationWidget",
        "imageChanged",
        "",
        "currentTabChanged",
        "index",
        "tabCloseRequested",
        "tabMoved",
        "fromIndex",
        "toIndex",
        "tabContextMenuOpened",
        "activateScale",
        "activateCrop",
        "activateRotate",
        "activateModifyCanvas",
        "activateCut",
        "persistDockWidgets",
        "restoreDockWidgetsState"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'imageChanged'
        QtMocHelpers::SignalData<void() const>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentTabChanged'
        QtMocHelpers::SignalData<void(int) const>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'tabCloseRequested'
        QtMocHelpers::SignalData<void(int) const>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'tabMoved'
        QtMocHelpers::SignalData<void(int, int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 },
        }}),
        // Signal 'tabContextMenuOpened'
        QtMocHelpers::SignalData<void(int) const>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'activateScale'
        QtMocHelpers::SignalData<void() const>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'activateCrop'
        QtMocHelpers::SignalData<void() const>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'activateRotate'
        QtMocHelpers::SignalData<void() const>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'activateModifyCanvas'
        QtMocHelpers::SignalData<void() const>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'activateCut'
        QtMocHelpers::SignalData<void() const>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'persistDockWidgets'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'restoreDockWidgetsState'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AnnotationWidget, qt_meta_tag_ZN15kImageAnnotator16AnnotationWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject kImageAnnotator::AnnotationWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator16AnnotationWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator16AnnotationWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15kImageAnnotator16AnnotationWidgetE_t>.metaTypes,
    nullptr
} };

void kImageAnnotator::AnnotationWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AnnotationWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->imageChanged(); break;
        case 1: _t->currentTabChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->tabCloseRequested((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->tabMoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->tabContextMenuOpened((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->activateScale(); break;
        case 6: _t->activateCrop(); break;
        case 7: _t->activateRotate(); break;
        case 8: _t->activateModifyCanvas(); break;
        case 9: _t->activateCut(); break;
        case 10: _t->persistDockWidgets(); break;
        case 11: _t->restoreDockWidgetsState(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AnnotationWidget::*)() const>(_a, &AnnotationWidget::imageChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationWidget::*)(int ) const>(_a, &AnnotationWidget::currentTabChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationWidget::*)(int ) const>(_a, &AnnotationWidget::tabCloseRequested, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationWidget::*)(int , int )>(_a, &AnnotationWidget::tabMoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationWidget::*)(int ) const>(_a, &AnnotationWidget::tabContextMenuOpened, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationWidget::*)() const>(_a, &AnnotationWidget::activateScale, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationWidget::*)() const>(_a, &AnnotationWidget::activateCrop, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationWidget::*)() const>(_a, &AnnotationWidget::activateRotate, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationWidget::*)() const>(_a, &AnnotationWidget::activateModifyCanvas, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationWidget::*)() const>(_a, &AnnotationWidget::activateCut, 9))
            return;
    }
}

const QMetaObject *kImageAnnotator::AnnotationWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *kImageAnnotator::AnnotationWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator16AnnotationWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int kImageAnnotator::AnnotationWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void kImageAnnotator::AnnotationWidget::imageChanged()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::AnnotationWidget *>(this), &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void kImageAnnotator::AnnotationWidget::currentTabChanged(int _t1)const
{
    QMetaObject::activate<void>(const_cast< kImageAnnotator::AnnotationWidget *>(this), &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void kImageAnnotator::AnnotationWidget::tabCloseRequested(int _t1)const
{
    QMetaObject::activate<void>(const_cast< kImageAnnotator::AnnotationWidget *>(this), &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void kImageAnnotator::AnnotationWidget::tabMoved(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void kImageAnnotator::AnnotationWidget::tabContextMenuOpened(int _t1)const
{
    QMetaObject::activate<void>(const_cast< kImageAnnotator::AnnotationWidget *>(this), &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void kImageAnnotator::AnnotationWidget::activateScale()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::AnnotationWidget *>(this), &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void kImageAnnotator::AnnotationWidget::activateCrop()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::AnnotationWidget *>(this), &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void kImageAnnotator::AnnotationWidget::activateRotate()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::AnnotationWidget *>(this), &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void kImageAnnotator::AnnotationWidget::activateModifyCanvas()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::AnnotationWidget *>(this), &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void kImageAnnotator::AnnotationWidget::activateCut()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::AnnotationWidget *>(this), &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
