/****************************************************************************
** Meta object code from reading C++ file 'AnnotationItemSettings.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../kImageAnnotator/src/gui/annotator/settings/AnnotationItemSettings.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AnnotationItemSettings.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15kImageAnnotator22AnnotationItemSettingsE_t {};
} // unnamed namespace

template <> constexpr inline auto kImageAnnotator::AnnotationItemSettings::qt_create_metaobjectdata<qt_meta_tag_ZN15kImageAnnotator22AnnotationItemSettingsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "kImageAnnotator::AnnotationItemSettings",
        "toolColorChanged",
        "",
        "QColor",
        "color",
        "toolTextColorChanged",
        "toolWidthChanged",
        "width",
        "toolFillTypeChanged",
        "FillModes",
        "fill",
        "notifyNumberToolSeedChanged",
        "newNumberToolSeed",
        "obfuscateFactorChanged",
        "factor",
        "stickerChanged",
        "sticker",
        "shadowEnabledChanged",
        "enabled",
        "fontChanged",
        "QFont",
        "font",
        "scalingChanged",
        "opacity",
        "opacityChanged",
        "scalingSelected",
        "scaling",
        "opacitySelected"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'toolColorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'toolTextColorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'toolWidthChanged'
        QtMocHelpers::SignalData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'toolFillTypeChanged'
        QtMocHelpers::SignalData<void(FillModes)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'notifyNumberToolSeedChanged'
        QtMocHelpers::SignalData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Signal 'obfuscateFactorChanged'
        QtMocHelpers::SignalData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Signal 'stickerChanged'
        QtMocHelpers::SignalData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Signal 'shadowEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Signal 'fontChanged'
        QtMocHelpers::SignalData<void(const QFont &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Signal 'scalingChanged'
        QtMocHelpers::SignalData<void(qreal)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 23 },
        }}),
        // Signal 'opacityChanged'
        QtMocHelpers::SignalData<void(qreal)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 23 },
        }}),
        // Slot 'scalingSelected'
        QtMocHelpers::SlotData<void(int)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 26 },
        }}),
        // Slot 'opacitySelected'
        QtMocHelpers::SlotData<void(int)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AnnotationItemSettings, qt_meta_tag_ZN15kImageAnnotator22AnnotationItemSettingsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject kImageAnnotator::AnnotationItemSettings::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractAnnotationDockWidgetContent::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator22AnnotationItemSettingsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator22AnnotationItemSettingsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15kImageAnnotator22AnnotationItemSettingsE_t>.metaTypes,
    nullptr
} };

void kImageAnnotator::AnnotationItemSettings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AnnotationItemSettings *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->toolColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 1: _t->toolTextColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 2: _t->toolWidthChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->toolFillTypeChanged((*reinterpret_cast<std::add_pointer_t<FillModes>>(_a[1]))); break;
        case 4: _t->notifyNumberToolSeedChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->obfuscateFactorChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->stickerChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->shadowEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->fontChanged((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 9: _t->scalingChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 10: _t->opacityChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 11: _t->scalingSelected((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->opacitySelected((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AnnotationItemSettings::*)(const QColor & )>(_a, &AnnotationItemSettings::toolColorChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationItemSettings::*)(const QColor & )>(_a, &AnnotationItemSettings::toolTextColorChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationItemSettings::*)(int )>(_a, &AnnotationItemSettings::toolWidthChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationItemSettings::*)(FillModes )>(_a, &AnnotationItemSettings::toolFillTypeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationItemSettings::*)(int )>(_a, &AnnotationItemSettings::notifyNumberToolSeedChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationItemSettings::*)(int )>(_a, &AnnotationItemSettings::obfuscateFactorChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationItemSettings::*)(const QString & )>(_a, &AnnotationItemSettings::stickerChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationItemSettings::*)(bool )>(_a, &AnnotationItemSettings::shadowEnabledChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationItemSettings::*)(const QFont & )>(_a, &AnnotationItemSettings::fontChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationItemSettings::*)(qreal )>(_a, &AnnotationItemSettings::scalingChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationItemSettings::*)(qreal )>(_a, &AnnotationItemSettings::opacityChanged, 10))
            return;
    }
}

const QMetaObject *kImageAnnotator::AnnotationItemSettings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *kImageAnnotator::AnnotationItemSettings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator22AnnotationItemSettingsE_t>.strings))
        return static_cast<void*>(this);
    return AbstractAnnotationDockWidgetContent::qt_metacast(_clname);
}

int kImageAnnotator::AnnotationItemSettings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractAnnotationDockWidgetContent::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void kImageAnnotator::AnnotationItemSettings::toolColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void kImageAnnotator::AnnotationItemSettings::toolTextColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void kImageAnnotator::AnnotationItemSettings::toolWidthChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void kImageAnnotator::AnnotationItemSettings::toolFillTypeChanged(FillModes _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void kImageAnnotator::AnnotationItemSettings::notifyNumberToolSeedChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void kImageAnnotator::AnnotationItemSettings::obfuscateFactorChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void kImageAnnotator::AnnotationItemSettings::stickerChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void kImageAnnotator::AnnotationItemSettings::shadowEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void kImageAnnotator::AnnotationItemSettings::fontChanged(const QFont & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void kImageAnnotator::AnnotationItemSettings::scalingChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void kImageAnnotator::AnnotationItemSettings::opacityChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}
QT_WARNING_POP
