/****************************************************************************
** Meta object code from reading C++ file 'Config.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../kImageAnnotator/src/backend/Config.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Config.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15kImageAnnotator6ConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto kImageAnnotator::Config::qt_create_metaobjectdata<qt_meta_tag_ZN15kImageAnnotator6ConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "kImageAnnotator::Config",
        "numberUpdateModeChanged",
        "",
        "NumberUpdateMode",
        "numberUpdateMode",
        "selectedTool",
        "Tools",
        "setSelectedToolType",
        "tool",
        "toolColor",
        "QColor",
        "setToolColor",
        "color",
        "toolTextColor",
        "setToolTextColor",
        "toolWidth",
        "setToolWidth",
        "width",
        "toolFillType",
        "FillModes",
        "setToolFillType",
        "fillType",
        "toolFont",
        "QFont",
        "setToolFont",
        "font",
        "shadowEnabled",
        "setShadowEnabled",
        "enabled",
        "smoothPathEnabled",
        "setSmoothPathEnabled",
        "setSaveToolSelection",
        "smoothFactor",
        "setSmoothFactor",
        "factor",
        "switchToSelectToolAfterDrawingItem",
        "setSwitchToSelectToolAfterDrawingItem",
        "setNumberToolUpdateMode",
        "obfuscationFactor",
        "toolType",
        "setObfuscationFactor",
        "canvasColor",
        "setCanvasColor",
        "annotatorDockWidgetsState",
        "setAnnotatorDockWidgetsState",
        "state",
        "selectItemAfterDrawing",
        "setSelectItemAfterDrawing",
        "toolScaling",
        "setToolScaling",
        "scaling",
        "toolOpacity",
        "setToolOpacity",
        "opacity"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'numberUpdateModeChanged'
        QtMocHelpers::SignalData<void(NumberUpdateMode) const>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'selectedTool'
        QtMocHelpers::SlotData<Tools() const>(5, 2, QMC::AccessPublic, 0x80000000 | 6),
        // Slot 'setSelectedToolType'
        QtMocHelpers::SlotData<void(Tools)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 8 },
        }}),
        // Slot 'toolColor'
        QtMocHelpers::SlotData<QColor(Tools) const>(9, 2, QMC::AccessPublic, 0x80000000 | 10, {{
            { 0x80000000 | 6, 8 },
        }}),
        // Slot 'setToolColor'
        QtMocHelpers::SlotData<void(const QColor &, Tools)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 12 }, { 0x80000000 | 6, 8 },
        }}),
        // Slot 'toolTextColor'
        QtMocHelpers::SlotData<QColor(Tools) const>(13, 2, QMC::AccessPublic, 0x80000000 | 10, {{
            { 0x80000000 | 6, 8 },
        }}),
        // Slot 'setToolTextColor'
        QtMocHelpers::SlotData<void(const QColor &, Tools)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 12 }, { 0x80000000 | 6, 8 },
        }}),
        // Slot 'toolWidth'
        QtMocHelpers::SlotData<int(Tools) const>(15, 2, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 6, 8 },
        }}),
        // Slot 'setToolWidth'
        QtMocHelpers::SlotData<void(int, Tools)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 17 }, { 0x80000000 | 6, 8 },
        }}),
        // Slot 'toolFillType'
        QtMocHelpers::SlotData<FillModes(Tools) const>(18, 2, QMC::AccessPublic, 0x80000000 | 19, {{
            { 0x80000000 | 6, 8 },
        }}),
        // Slot 'setToolFillType'
        QtMocHelpers::SlotData<void(FillModes, Tools)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 21 }, { 0x80000000 | 6, 8 },
        }}),
        // Slot 'toolFont'
        QtMocHelpers::SlotData<QFont(Tools) const>(22, 2, QMC::AccessPublic, 0x80000000 | 23, {{
            { 0x80000000 | 6, 8 },
        }}),
        // Slot 'setToolFont'
        QtMocHelpers::SlotData<void(const QFont &, Tools)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 25 }, { 0x80000000 | 6, 8 },
        }}),
        // Slot 'shadowEnabled'
        QtMocHelpers::SlotData<bool(Tools) const>(26, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 6, 8 },
        }}),
        // Slot 'setShadowEnabled'
        QtMocHelpers::SlotData<void(bool, Tools)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 28 }, { 0x80000000 | 6, 8 },
        }}),
        // Slot 'smoothPathEnabled'
        QtMocHelpers::SlotData<bool() const>(29, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setSmoothPathEnabled'
        QtMocHelpers::SlotData<void(bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 28 },
        }}),
        // Slot 'setSaveToolSelection'
        QtMocHelpers::SlotData<void(bool)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 28 },
        }}),
        // Slot 'smoothFactor'
        QtMocHelpers::SlotData<int() const>(32, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'setSmoothFactor'
        QtMocHelpers::SlotData<void(int)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 34 },
        }}),
        // Slot 'switchToSelectToolAfterDrawingItem'
        QtMocHelpers::SlotData<bool() const>(35, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setSwitchToSelectToolAfterDrawingItem'
        QtMocHelpers::SlotData<void(bool)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 28 },
        }}),
        // Slot 'numberUpdateMode'
        QtMocHelpers::SlotData<NumberUpdateMode() const>(4, 2, QMC::AccessPublic, 0x80000000 | 3),
        // Slot 'setNumberToolUpdateMode'
        QtMocHelpers::SlotData<void(NumberUpdateMode)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'obfuscationFactor'
        QtMocHelpers::SlotData<int(Tools) const>(38, 2, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 6, 39 },
        }}),
        // Slot 'setObfuscationFactor'
        QtMocHelpers::SlotData<void(int, Tools)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 34 }, { 0x80000000 | 6, 39 },
        }}),
        // Slot 'canvasColor'
        QtMocHelpers::SlotData<QColor() const>(41, 2, QMC::AccessPublic, 0x80000000 | 10),
        // Slot 'setCanvasColor'
        QtMocHelpers::SlotData<void(const QColor &)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 12 },
        }}),
        // Slot 'annotatorDockWidgetsState'
        QtMocHelpers::SlotData<QByteArray() const>(43, 2, QMC::AccessPublic, QMetaType::QByteArray),
        // Slot 'setAnnotatorDockWidgetsState'
        QtMocHelpers::SlotData<void(const QByteArray &)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 45 },
        }}),
        // Slot 'selectItemAfterDrawing'
        QtMocHelpers::SlotData<bool() const>(46, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setSelectItemAfterDrawing'
        QtMocHelpers::SlotData<void(bool)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 28 },
        }}),
        // Slot 'toolScaling'
        QtMocHelpers::SlotData<qreal(Tools) const>(48, 2, QMC::AccessPublic, QMetaType::QReal, {{
            { 0x80000000 | 6, 8 },
        }}),
        // Slot 'setToolScaling'
        QtMocHelpers::SlotData<void(qreal, Tools)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 50 }, { 0x80000000 | 6, 8 },
        }}),
        // Slot 'toolOpacity'
        QtMocHelpers::SlotData<qreal(Tools) const>(51, 2, QMC::AccessPublic, QMetaType::QReal, {{
            { 0x80000000 | 6, 8 },
        }}),
        // Slot 'setToolOpacity'
        QtMocHelpers::SlotData<void(qreal, Tools)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 53 }, { 0x80000000 | 6, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Config, qt_meta_tag_ZN15kImageAnnotator6ConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject kImageAnnotator::Config::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator6ConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator6ConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15kImageAnnotator6ConfigE_t>.metaTypes,
    nullptr
} };

void kImageAnnotator::Config::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Config *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->numberUpdateModeChanged((*reinterpret_cast<std::add_pointer_t<NumberUpdateMode>>(_a[1]))); break;
        case 1: { Tools _r = _t->selectedTool();
            if (_a[0]) *reinterpret_cast<Tools*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->setSelectedToolType((*reinterpret_cast<std::add_pointer_t<Tools>>(_a[1]))); break;
        case 3: { QColor _r = _t->toolColor((*reinterpret_cast<std::add_pointer_t<Tools>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QColor*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->setToolColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Tools>>(_a[2]))); break;
        case 5: { QColor _r = _t->toolTextColor((*reinterpret_cast<std::add_pointer_t<Tools>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QColor*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->setToolTextColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Tools>>(_a[2]))); break;
        case 7: { int _r = _t->toolWidth((*reinterpret_cast<std::add_pointer_t<Tools>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->setToolWidth((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Tools>>(_a[2]))); break;
        case 9: { FillModes _r = _t->toolFillType((*reinterpret_cast<std::add_pointer_t<Tools>>(_a[1])));
            if (_a[0]) *reinterpret_cast<FillModes*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->setToolFillType((*reinterpret_cast<std::add_pointer_t<FillModes>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Tools>>(_a[2]))); break;
        case 11: { QFont _r = _t->toolFont((*reinterpret_cast<std::add_pointer_t<Tools>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QFont*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->setToolFont((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Tools>>(_a[2]))); break;
        case 13: { bool _r = _t->shadowEnabled((*reinterpret_cast<std::add_pointer_t<Tools>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->setShadowEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Tools>>(_a[2]))); break;
        case 15: { bool _r = _t->smoothPathEnabled();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: _t->setSmoothPathEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->setSaveToolSelection((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: { int _r = _t->smoothFactor();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 19: _t->setSmoothFactor((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: { bool _r = _t->switchToSelectToolAfterDrawingItem();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: _t->setSwitchToSelectToolAfterDrawingItem((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: { NumberUpdateMode _r = _t->numberUpdateMode();
            if (_a[0]) *reinterpret_cast<NumberUpdateMode*>(_a[0]) = std::move(_r); }  break;
        case 23: _t->setNumberToolUpdateMode((*reinterpret_cast<std::add_pointer_t<NumberUpdateMode>>(_a[1]))); break;
        case 24: { int _r = _t->obfuscationFactor((*reinterpret_cast<std::add_pointer_t<Tools>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 25: _t->setObfuscationFactor((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Tools>>(_a[2]))); break;
        case 26: { QColor _r = _t->canvasColor();
            if (_a[0]) *reinterpret_cast<QColor*>(_a[0]) = std::move(_r); }  break;
        case 27: _t->setCanvasColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 28: { QByteArray _r = _t->annotatorDockWidgetsState();
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 29: _t->setAnnotatorDockWidgetsState((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 30: { bool _r = _t->selectItemAfterDrawing();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 31: _t->setSelectItemAfterDrawing((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 32: { qreal _r = _t->toolScaling((*reinterpret_cast<std::add_pointer_t<Tools>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qreal*>(_a[0]) = std::move(_r); }  break;
        case 33: _t->setToolScaling((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Tools>>(_a[2]))); break;
        case 34: { qreal _r = _t->toolOpacity((*reinterpret_cast<std::add_pointer_t<Tools>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qreal*>(_a[0]) = std::move(_r); }  break;
        case 35: _t->setToolOpacity((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Tools>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Config::*)(NumberUpdateMode ) const>(_a, &Config::numberUpdateModeChanged, 0))
            return;
    }
}

const QMetaObject *kImageAnnotator::Config::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *kImageAnnotator::Config::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator6ConfigE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int kImageAnnotator::Config::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 36;
    }
    return _id;
}

// SIGNAL 0
void kImageAnnotator::Config::numberUpdateModeChanged(NumberUpdateMode _t1)const
{
    QMetaObject::activate<void>(const_cast< kImageAnnotator::Config *>(this), &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
