/****************************************************************************
** Meta object code from reading C++ file 'AnnotationContextMenu.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../kImageAnnotator/src/annotations/misc/AnnotationContextMenu.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AnnotationContextMenu.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15kImageAnnotator21AnnotationContextMenuE_t {};
} // unnamed namespace

template <> constexpr inline auto kImageAnnotator::AnnotationContextMenu::qt_create_metaobjectdata<qt_meta_tag_ZN15kImageAnnotator21AnnotationContextMenuE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "kImageAnnotator::AnnotationContextMenu",
        "bringToFront",
        "",
        "bringForward",
        "sendBackward",
        "sendToBack",
        "copy",
        "QPointF",
        "position",
        "paste",
        "erase",
        "edit"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'bringToFront'
        QtMocHelpers::SignalData<void() const>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bringForward'
        QtMocHelpers::SignalData<void() const>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sendBackward'
        QtMocHelpers::SignalData<void() const>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sendToBack'
        QtMocHelpers::SignalData<void() const>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'copy'
        QtMocHelpers::SignalData<void(const QPointF &) const>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'paste'
        QtMocHelpers::SignalData<void(const QPointF &) const>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'erase'
        QtMocHelpers::SignalData<void() const>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'edit'
        QtMocHelpers::SignalData<void() const>(11, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AnnotationContextMenu, qt_meta_tag_ZN15kImageAnnotator21AnnotationContextMenuE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject kImageAnnotator::AnnotationContextMenu::staticMetaObject = { {
    QMetaObject::SuperData::link<QMenu::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator21AnnotationContextMenuE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator21AnnotationContextMenuE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15kImageAnnotator21AnnotationContextMenuE_t>.metaTypes,
    nullptr
} };

void kImageAnnotator::AnnotationContextMenu::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AnnotationContextMenu *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->bringToFront(); break;
        case 1: _t->bringForward(); break;
        case 2: _t->sendBackward(); break;
        case 3: _t->sendToBack(); break;
        case 4: _t->copy((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 5: _t->paste((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 6: _t->erase(); break;
        case 7: _t->edit(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AnnotationContextMenu::*)() const>(_a, &AnnotationContextMenu::bringToFront, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationContextMenu::*)() const>(_a, &AnnotationContextMenu::bringForward, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationContextMenu::*)() const>(_a, &AnnotationContextMenu::sendBackward, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationContextMenu::*)() const>(_a, &AnnotationContextMenu::sendToBack, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationContextMenu::*)(const QPointF & ) const>(_a, &AnnotationContextMenu::copy, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationContextMenu::*)(const QPointF & ) const>(_a, &AnnotationContextMenu::paste, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationContextMenu::*)() const>(_a, &AnnotationContextMenu::erase, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationContextMenu::*)() const>(_a, &AnnotationContextMenu::edit, 7))
            return;
    }
}

const QMetaObject *kImageAnnotator::AnnotationContextMenu::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *kImageAnnotator::AnnotationContextMenu::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator21AnnotationContextMenuE_t>.strings))
        return static_cast<void*>(this);
    return QMenu::qt_metacast(_clname);
}

int kImageAnnotator::AnnotationContextMenu::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMenu::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void kImageAnnotator::AnnotationContextMenu::bringToFront()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::AnnotationContextMenu *>(this), &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void kImageAnnotator::AnnotationContextMenu::bringForward()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::AnnotationContextMenu *>(this), &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void kImageAnnotator::AnnotationContextMenu::sendBackward()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::AnnotationContextMenu *>(this), &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void kImageAnnotator::AnnotationContextMenu::sendToBack()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::AnnotationContextMenu *>(this), &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void kImageAnnotator::AnnotationContextMenu::copy(const QPointF & _t1)const
{
    QMetaObject::activate<void>(const_cast< kImageAnnotator::AnnotationContextMenu *>(this), &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void kImageAnnotator::AnnotationContextMenu::paste(const QPointF & _t1)const
{
    QMetaObject::activate<void>(const_cast< kImageAnnotator::AnnotationContextMenu *>(this), &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void kImageAnnotator::AnnotationContextMenu::erase()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::AnnotationContextMenu *>(this), &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void kImageAnnotator::AnnotationContextMenu::edit()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::AnnotationContextMenu *>(this), &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
