/****************************************************************************
** Meta object code from reading C++ file 'AnnotationTabContextMenu.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../kImageAnnotator/src/gui/annotator/tabs/AnnotationTabContextMenu.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AnnotationTabContextMenu.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15kImageAnnotator24AnnotationTabContextMenuE_t {};
} // unnamed namespace

template <> constexpr inline auto kImageAnnotator::AnnotationTabContextMenu::qt_create_metaobjectdata<qt_meta_tag_ZN15kImageAnnotator24AnnotationTabContextMenuE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "kImageAnnotator::AnnotationTabContextMenu",
        "closeTab",
        "",
        "index",
        "closeOtherTabs",
        "closeAllTabs",
        "closeAllTabsToLeft",
        "closeAllTabsToRight",
        "customActionTriggered",
        "customActionChanged",
        "addCustomAction",
        "QAction*",
        "innerAction"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'closeTab'
        QtMocHelpers::SignalData<void(int) const>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'closeOtherTabs'
        QtMocHelpers::SignalData<void(int) const>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'closeAllTabs'
        QtMocHelpers::SignalData<void() const>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'closeAllTabsToLeft'
        QtMocHelpers::SignalData<void(int) const>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'closeAllTabsToRight'
        QtMocHelpers::SignalData<void(int) const>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'customActionTriggered'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'customActionChanged'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'addCustomAction'
        QtMocHelpers::SlotData<void(QAction *)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AnnotationTabContextMenu, qt_meta_tag_ZN15kImageAnnotator24AnnotationTabContextMenuE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject kImageAnnotator::AnnotationTabContextMenu::staticMetaObject = { {
    QMetaObject::SuperData::link<QMenu::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator24AnnotationTabContextMenuE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator24AnnotationTabContextMenuE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15kImageAnnotator24AnnotationTabContextMenuE_t>.metaTypes,
    nullptr
} };

void kImageAnnotator::AnnotationTabContextMenu::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AnnotationTabContextMenu *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->closeTab((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->closeOtherTabs((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->closeAllTabs(); break;
        case 3: _t->closeAllTabsToLeft((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->closeAllTabsToRight((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->customActionTriggered(); break;
        case 6: _t->customActionChanged(); break;
        case 7: _t->addCustomAction((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AnnotationTabContextMenu::*)(int ) const>(_a, &AnnotationTabContextMenu::closeTab, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationTabContextMenu::*)(int ) const>(_a, &AnnotationTabContextMenu::closeOtherTabs, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationTabContextMenu::*)() const>(_a, &AnnotationTabContextMenu::closeAllTabs, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationTabContextMenu::*)(int ) const>(_a, &AnnotationTabContextMenu::closeAllTabsToLeft, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnnotationTabContextMenu::*)(int ) const>(_a, &AnnotationTabContextMenu::closeAllTabsToRight, 4))
            return;
    }
}

const QMetaObject *kImageAnnotator::AnnotationTabContextMenu::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *kImageAnnotator::AnnotationTabContextMenu::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator24AnnotationTabContextMenuE_t>.strings))
        return static_cast<void*>(this);
    return QMenu::qt_metacast(_clname);
}

int kImageAnnotator::AnnotationTabContextMenu::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMenu::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void kImageAnnotator::AnnotationTabContextMenu::closeTab(int _t1)const
{
    QMetaObject::activate<void>(const_cast< kImageAnnotator::AnnotationTabContextMenu *>(this), &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void kImageAnnotator::AnnotationTabContextMenu::closeOtherTabs(int _t1)const
{
    QMetaObject::activate<void>(const_cast< kImageAnnotator::AnnotationTabContextMenu *>(this), &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void kImageAnnotator::AnnotationTabContextMenu::closeAllTabs()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::AnnotationTabContextMenu *>(this), &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void kImageAnnotator::AnnotationTabContextMenu::closeAllTabsToLeft(int _t1)const
{
    QMetaObject::activate<void>(const_cast< kImageAnnotator::AnnotationTabContextMenu *>(this), &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void kImageAnnotator::AnnotationTabContextMenu::closeAllTabsToRight(int _t1)const
{
    QMetaObject::activate<void>(const_cast< kImageAnnotator::AnnotationTabContextMenu *>(this), &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
