/*
 * Copyright (C) 2020 Damir Porobic <damir.porobic@gmx.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "GridMenuToolButton.h"

namespace kImageAnnotator {

GridMenuToolButton::GridMenuToolButton(QWidget *parent) :
	QToolButton(parent),
	mMenu(new GridMenu(this))
{
	setMenu(mMenu);

	connect(mMenu, &GridMenu::selectionChanged, this, &GridMenuToolButton::selectionChanged);
	connect(mMenu, &GridMenu::selectionChanged, this, &GridMenuToolButton::updateIconAndToolTip);
}

GridMenuToolButton::~GridMenuToolButton()
{
	delete mMenu;
}

void GridMenuToolButton::addItem(const QIcon &icon, const QString &toolTip, const QVariant &data)
{
	mMenu->addItem(icon, toolTip, data);
}

QVariant GridMenuToolButton::currentData() const
{
	return mMenu->currentData();
}

void kImageAnnotator::GridMenuToolButton::setCurrentData(const QVariant &data)
{
	mMenu->setCurrentData(data);
}

void GridMenuToolButton::clear()
{
	mMenu->clear();
}

void GridMenuToolButton::updateIconAndToolTip()
{
	setIcon(mMenu->currentIcon());
	setToolTip(mMenu->currentToolTip());
}

} // namespace kImageAnnotator