// This file is autogenerated. Changes will be overwritten.
#include "HJ3KT3DVXG/moc_KImageAnnotator.cpp"
#include "V6IDEG75QY/moc_AnnotationArea.cpp"
#include "V6IDEG75QY/moc_AnnotationItemFactory.cpp"
#include "V6IDEG75QY/moc_ZoomValueProvider.cpp"
#include "KY6PQFXFAJ/moc_BorderImageEffect.cpp"
#include "KY6PQFXFAJ/moc_DropShadowImageEffect.cpp"
#include "KY6PQFXFAJ/moc_GrayscaleImageEffect.cpp"
#include "KY6PQFXFAJ/moc_InvertColorImageEffect.cpp"
#include "KY6PQFXFAJ/moc_NoImageEffect.cpp"
#include "YL5MH5TTTQ/moc_AbstractAnnotationItem.cpp"
#include "YL5MH5TTTQ/moc_AnnotationMarkerEllipse.cpp"
#include "YL5MH5TTTQ/moc_AnnotationMarkerPen.cpp"
#include "YL5MH5TTTQ/moc_AnnotationMarkerRect.cpp"
#include "YL5MH5TTTQ/moc_AnnotationNumber.cpp"
#include "YL5MH5TTTQ/moc_AnnotationNumberArrow.cpp"
#include "YL5MH5TTTQ/moc_AnnotationNumberPointer.cpp"
#include "YL5MH5TTTQ/moc_AnnotationRect.cpp"
#include "YL5MH5TTTQ/moc_AnnotationText.cpp"
#include "YL5MH5TTTQ/moc_AnnotationTextArrow.cpp"
#include "YL5MH5TTTQ/moc_AnnotationTextPointer.cpp"
#include "AJBCAZNY4V/moc_AnnotationTextHandler.cpp"
#include "AJBCAZNY4V/moc_KeyInputHelper.cpp"
#include "AJBCAZNY4V/moc_TextCursor.cpp"
#include "AJBCAZNY4V/moc_TextHandlerItem.cpp"
#include "SOWL6Y5B35/moc_AnnotationContextMenu.cpp"
#include "SOWL6Y5B35/moc_AnnotationItemClipboard.cpp"
#include "SOWL6Y5B35/moc_NumberManager.cpp"
#include "3QWSO7TZIT/moc_AnnotationItemArranger.cpp"
#include "3QWSO7TZIT/moc_AnnotationItemEditor.cpp"
#include "3QWSO7TZIT/moc_AnnotationItemModifier.cpp"
#include "3QWSO7TZIT/moc_AnnotationItemMover.cpp"
#include "3QWSO7TZIT/moc_AnnotationItemResizer.cpp"
#include "3QWSO7TZIT/moc_AnnotationMultiItemResizer.cpp"
#include "VU2PALRVTW/moc_Config.cpp"
#include "JNC5TJ7Y4G/moc_IgnoreShortcutsFilter.cpp"
#include "JNC5TJ7Y4G/moc_KeyEventFilter.cpp"
#include "QUNZOKWISN/moc_KeyHelper.cpp"
#include "MXUWEOXILK/moc_CoreView.cpp"
#include "OGTGIAJO2T/moc_AnnotationView.cpp"
#include "OGTGIAJO2T/moc_AnnotationWidget.cpp"
#include "CNGFO4VNMZ/moc_AnnotationDockWidget.cpp"
#include "CNGFO4VNMZ/moc_AnnotationDockWidgetDragHandle.cpp"
#include "TIGL6BMEIT/moc_AnnotationControlsWidget.cpp"
#include "TIGL6BMEIT/moc_AnnotationGeneralSettings.cpp"
#include "TIGL6BMEIT/moc_AnnotationImageSettings.cpp"
#include "TIGL6BMEIT/moc_AnnotationItemSettings.cpp"
#include "TIGL6BMEIT/moc_AnnotationSettingsAdapter.cpp"
#include "TIGL6BMEIT/moc_AnnotationToolSelection.cpp"
#include "RRX6JJRTLQ/moc_AnnotationTabClickEventFilter.cpp"
#include "RRX6JJRTLQ/moc_AnnotationTabCloser.cpp"
#include "RRX6JJRTLQ/moc_AnnotationTabContent.cpp"
#include "RRX6JJRTLQ/moc_AnnotationTabContextMenu.cpp"
#include "RRX6JJRTLQ/moc_AnnotationTabWidget.cpp"
#include "JVHKFIUBQP/moc_ModifyCanvasView.cpp"
#include "JVHKFIUBQP/moc_ModifyCanvasWidget.cpp"
#include "LDXNBURBEE/moc_CropView.cpp"
#include "LDXNBURBEE/moc_CropWidget.cpp"
#include "QSVVTWKBP7/moc_CutView.cpp"
#include "QSVVTWKBP7/moc_CutWidget.cpp"
#include "5XSCTNUIO4/moc_RotateDialog.cpp"
#include "5XSCTNUIO4/moc_RotateWidget.cpp"
#include "4EJABM3VU3/moc_ScaleDialog.cpp"
#include "4EJABM3VU3/moc_ScaleSizeHandler.cpp"
#include "4EJABM3VU3/moc_ScaleWidget.cpp"
#include "4GKJYAD4B4/moc_ScrollAndZoomView.cpp"
#include "4GKJYAD4B4/moc_ViewZoomer.cpp"
#include "RUBIVWG3AB/moc_BaseSelectionView.cpp"
#include "RUBIVWG3AB/moc_SelectionHandler.cpp"
#include "RZ4CM6YYJ6/moc_ColorDialogButton.cpp"
#include "RZ4CM6YYJ6/moc_Controls.cpp"
#include "RZ4CM6YYJ6/moc_CustomFontComboBox.cpp"
#include "RZ4CM6YYJ6/moc_CustomSpinBox.cpp"
#include "RZ4CM6YYJ6/moc_CustomToolButton.cpp"
#include "RZ4CM6YYJ6/moc_ToggleButton.cpp"
#include "RZ4CM6YYJ6/moc_ToolPicker.cpp"
#include "CCYIKDJIXN/moc_GridMenu.cpp"
#include "CCYIKDJIXN/moc_GridMenuButton.cpp"
#include "CCYIKDJIXN/moc_GridMenuToolButton.cpp"
#include "CCYIKDJIXN/moc_ListItemGroup.cpp"
#include "CCYIKDJIXN/moc_ListMenu.cpp"
#include "CCYIKDJIXN/moc_ListMenuItem.cpp"
#include "CCYIKDJIXN/moc_ListMenuToolButton.cpp"
#include "GZV54CKLPE/moc_AttachedSeparator.cpp"
#include "SS2GKST3NV/moc_BoolPicker.cpp"
#include "SS2GKST3NV/moc_ColorPicker.cpp"
#include "SS2GKST3NV/moc_FillModePicker.cpp"
#include "SS2GKST3NV/moc_FontPicker.cpp"
#include "SS2GKST3NV/moc_ImageEffectPicker.cpp"
#include "SS2GKST3NV/moc_NumberPicker.cpp"
#include "SS2GKST3NV/moc_SettingsPickerWidget.cpp"
#include "SS2GKST3NV/moc_StickerPicker.cpp"
#include "SS2GKST3NV/moc_ZoomPicker.cpp"
