/****************************************************************************
** Meta object code from reading C++ file 'CoreView.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../kImageAnnotator/src/gui/CoreView.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'CoreView.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15kImageAnnotator8CoreViewE_t {};
} // unnamed namespace

template <> constexpr inline auto kImageAnnotator::CoreView::qt_create_metaobjectdata<qt_meta_tag_ZN15kImageAnnotator8CoreViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "kImageAnnotator::CoreView",
        "imageChanged",
        "",
        "currentTabChanged",
        "index",
        "tabCloseRequested",
        "tabMoved",
        "fromIndex",
        "toIndex",
        "tabContextMenuOpened",
        "loadImage",
        "QPixmap",
        "pixmap",
        "addTab",
        "title",
        "toolTip",
        "updateTabInfo",
        "insertImageItem",
        "QPointF",
        "position",
        "removeTab",
        "showAnnotator",
        "showCropper",
        "showScaler",
        "showRotator",
        "showCutter",
        "showCanvasModifier",
        "setControlsWidgetVisible",
        "enabled",
        "setSettingsCollapsed",
        "isCollapsed",
        "setTabBarAutoHide",
        "setStickers",
        "stickerPaths",
        "keepDefault",
        "addTabContextMenuActions",
        "QList<QAction*>",
        "actions"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'imageChanged'
        QtMocHelpers::SignalData<void() const>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentTabChanged'
        QtMocHelpers::SignalData<void(int) const>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'tabCloseRequested'
        QtMocHelpers::SignalData<void(int) const>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'tabMoved'
        QtMocHelpers::SignalData<void(int, int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 },
        }}),
        // Signal 'tabContextMenuOpened'
        QtMocHelpers::SignalData<void(int) const>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'loadImage'
        QtMocHelpers::SlotData<void(const QPixmap &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'addTab'
        QtMocHelpers::SlotData<int(const QPixmap &, const QString &, const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 11, 12 }, { QMetaType::QString, 14 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'updateTabInfo'
        QtMocHelpers::SlotData<void(int, const QString &, const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 }, { QMetaType::QString, 14 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'insertImageItem'
        QtMocHelpers::SlotData<void(const QPointF &, const QPixmap &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { 0x80000000 | 11, 12 },
        }}),
        // Slot 'removeTab'
        QtMocHelpers::SlotData<void(int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'showAnnotator'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showCropper'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showScaler'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showRotator'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showCutter'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showCanvasModifier'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setControlsWidgetVisible'
        QtMocHelpers::SlotData<void(bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 28 },
        }}),
        // Slot 'setSettingsCollapsed'
        QtMocHelpers::SlotData<void(bool)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 30 },
        }}),
        // Slot 'setTabBarAutoHide'
        QtMocHelpers::SlotData<void(bool)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 28 },
        }}),
        // Slot 'setStickers'
        QtMocHelpers::SlotData<void(const QStringList &, bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 33 }, { QMetaType::Bool, 34 },
        }}),
        // Slot 'addTabContextMenuActions'
        QtMocHelpers::SlotData<void(const QList<QAction*> &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CoreView, qt_meta_tag_ZN15kImageAnnotator8CoreViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject kImageAnnotator::CoreView::staticMetaObject = { {
    QMetaObject::SuperData::link<QStackedWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator8CoreViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator8CoreViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15kImageAnnotator8CoreViewE_t>.metaTypes,
    nullptr
} };

void kImageAnnotator::CoreView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CoreView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->imageChanged(); break;
        case 1: _t->currentTabChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->tabCloseRequested((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->tabMoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->tabContextMenuOpened((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->loadImage((*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[1]))); break;
        case 6: { int _r = _t->addTab((*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->updateTabInfo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 8: _t->insertImageItem((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[2]))); break;
        case 9: _t->removeTab((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->showAnnotator(); break;
        case 11: _t->showCropper(); break;
        case 12: _t->showScaler(); break;
        case 13: _t->showRotator(); break;
        case 14: _t->showCutter(); break;
        case 15: _t->showCanvasModifier(); break;
        case 16: _t->setControlsWidgetVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->setSettingsCollapsed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->setTabBarAutoHide((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->setStickers((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 20: _t->addTabContextMenuActions((*reinterpret_cast<std::add_pointer_t<QList<QAction*>>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QAction*> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CoreView::*)() const>(_a, &CoreView::imageChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CoreView::*)(int ) const>(_a, &CoreView::currentTabChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CoreView::*)(int ) const>(_a, &CoreView::tabCloseRequested, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CoreView::*)(int , int )>(_a, &CoreView::tabMoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (CoreView::*)(int ) const>(_a, &CoreView::tabContextMenuOpened, 4))
            return;
    }
}

const QMetaObject *kImageAnnotator::CoreView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *kImageAnnotator::CoreView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15kImageAnnotator8CoreViewE_t>.strings))
        return static_cast<void*>(this);
    return QStackedWidget::qt_metacast(_clname);
}

int kImageAnnotator::CoreView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStackedWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void kImageAnnotator::CoreView::imageChanged()const
{
    QMetaObject::activate(const_cast< kImageAnnotator::CoreView *>(this), &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void kImageAnnotator::CoreView::currentTabChanged(int _t1)const
{
    QMetaObject::activate<void>(const_cast< kImageAnnotator::CoreView *>(this), &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void kImageAnnotator::CoreView::tabCloseRequested(int _t1)const
{
    QMetaObject::activate<void>(const_cast< kImageAnnotator::CoreView *>(this), &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void kImageAnnotator::CoreView::tabMoved(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void kImageAnnotator::CoreView::tabContextMenuOpened(int _t1)const
{
    QMetaObject::activate<void>(const_cast< kImageAnnotator::CoreView *>(this), &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
