/*
 * Copyright (C) 2021 Damir Porobic <damir.porobic@gmx.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "AnnotationMarkerEllipse.h"

namespace kImageAnnotator {

AnnotationMarkerEllipse::AnnotationMarkerEllipse(const QPointF &startPosition, const PropertiesPtr &properties) :
	AnnotationEllipse(startPosition, properties)
{

}

Tools AnnotationMarkerEllipse::toolType() const
{
	return Tools::MarkerEllipse;
}

void AnnotationMarkerEllipse::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
	AnnotationEllipse::paint(painter, option, widget);
}

QPainter::CompositionMode AnnotationMarkerEllipse::compositionMode() const
{
	return QPainter::CompositionMode_Multiply;
}

void AnnotationMarkerEllipse::updateShadow()
{
	setGraphicsEffect(nullptr);
}

} // namespace kImageAnnotator